@@ -14,6 +14,4 @@
 
 """Tools related to evaluating models"""
 
-__all__ = [
-    "util",
-]
+__all__ = ["util"]
